@@ -4,7 +4,20 @@ use crate::{
     defs::{LengthType, SpareMemoryPolicy},
     errors::{CapacityError, CapacityErrorVal, InsertError, InsertErrorVal},
 };
-use core::{marker::PhantomData, mem, ptr, result::Result, slice};
+use core::{
+    marker::PhantomData,
+    mem,
+    ops::{Bound, RangeBounds},
+    ptr,
+    result::Result,
+    slice,
+};
+
+mod drain;
+pub use drain::*;
+
+mod retain;
+use retain::*;
 
 /// A continuous non-growable array with vector-like API.
 ///
@@ -1019,6 +1032,181 @@ where
         SM::init(last, 1);
         tmp
     }
+
+    /// Creates a draining iterator that removes the specified range in the array-vector
+    /// and yields the removed items.
+    ///
+    /// When the iterator is dropped, all elements in the range are removed from the array-vector,
+    /// even if the iterator was not fully consumed.
+    /// If the iterator is not dropped (with [`mem::forget`] for example),
+    /// it is unspecified how many elements are removed.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the starting point is greater than the end point or if the end point is greater
+    /// than the length of the vector.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use cds::array_vec;
+    /// let mut a = array_vec![5; usize; 1, 2, 3, 4, 5];
+    /// assert_eq!(a, [1, 2, 3, 4, 5]);
+    /// for (index, i) in a.drain(0..3).enumerate() {
+    ///     assert_eq!(index + 1, i);
+    /// }
+    /// assert_eq!(a, [4, 5]);
+    /// ```
+    ///
+    /// [`mem::forget`]: core::mem::forget
+    pub fn drain<R>(&mut self, range: R) -> Drain<'_, T, L, SM, C>
+    where
+        R: RangeBounds<usize>,
+    {
+        let end = match range.end_bound() {
+            Bound::Included(e) => e
+                .checked_add(1)
+                .unwrap_or_else(|| panic!("end bound overflows")),
+            Bound::Excluded(e) => *e,
+            Bound::Unbounded => self.len(),
+        };
+
+        if end > self.len() {
+            panic!("invalid end bound");
+        }
+
+        let start = match range.start_bound() {
+            Bound::Included(s) => *s,
+            Bound::Excluded(s) => s
+                .checked_add(1)
+                .unwrap_or_else(|| panic!("start bound overflows")),
+            Bound::Unbounded => 0,
+        };
+
+        if start > end {
+            panic!("invalid range");
+        }
+
+        unsafe {
+            let len = self.len();
+            let (iter, tail, tail_len) = if start < end {
+                // set `len` to reflect the head only
+                self.set_len(start);
+
+                (
+                    slice::from_raw_parts(self.as_ptr().add(start), end - start).iter(),
+                    L::new(end),
+                    L::new(len - end),
+                )
+            } else {
+                // empty drained range, mark it with an impossible combination of `tail/tail_len`
+                ((&[]).iter(), L::new(L::MAX), L::new(L::MAX))
+            };
+
+            Drain {
+                av: ptr::NonNull::new_unchecked(self),
+                iter,
+                tail,
+                tail_len,
+            }
+        }
+    }
+
+    /// Retains only the elements specified by the predicate.
+    ///
+    /// In other words, remove all elements `e` such that `f(&e)` returns `false`.
+    /// This method operates in place, visiting each element exactly once in the original order,
+    /// and preserves the order of the retained elements.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use cds::array_vec;
+    /// let mut a = array_vec![5; 0, 1, 2, 3, 4];
+    /// assert_eq!(a, [0, 1, 2, 3, 4]);
+    /// a.retain(|e| (*e & 1) != 0);
+    /// assert_eq!(a, [1, 3]);
+    /// ```
+    #[inline]
+    pub fn retain<F>(&mut self, mut f: F)
+    where
+        F: FnMut(&T) -> bool,
+    {
+        self.retain_mut(|e| f(e))
+    }
+
+    /// Retains only the elements specified by the predicate, passing a mutable reference to it.
+    ///
+    /// In other words, remove all elements `e` such that `f(&mut e)` returns `false`.
+    /// This method operates in place, visiting each element exactly once in the original order,
+    /// and preserves the order of the retained elements.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use cds::array_vec;
+    /// let mut a = array_vec![5; 0, 1, 2, 3, 4];
+    /// assert_eq!(a, [0, 1, 2, 3, 4]);
+    /// a.retain_mut(|e| if (*e & 1) == 0 {
+    ///    *e *= *e;
+    ///    true
+    /// } else {
+    ///    false
+    /// });
+    /// assert_eq!(a, [0, 4, 16]);
+    /// ```
+    pub fn retain_mut<F>(&mut self, mut f: F)
+    where
+        F: FnMut(&mut T) -> bool,
+    {
+        let len = self.len();
+
+        // set `len` to zero, to avoid double-drop of deleted items.
+        // `len` is restored by RetainGuard.
+        unsafe { self.set_len(0) };
+
+        let mut g = RetainGuard {
+            av: self,
+            len,
+            deleted: 0,
+            processed: 0,
+        };
+
+        unsafe {
+            // no empty slot found yet, so there is nothing to move
+            while g.processed < len {
+                let item_mut_ref = &mut *g.av.as_mut_ptr().add(g.processed);
+                if !f(item_mut_ref) {
+                    // update counters before drop_in_place, as it may panic
+                    g.processed += 1;
+                    g.deleted += 1;
+                    ptr::drop_in_place(item_mut_ref);
+                    break;
+                }
+                g.processed += 1;
+            }
+
+            // If there are items left to process, there must be an empty slot.
+            // Move every retained slot to an empty one.
+            while g.processed < len {
+                let item_mut_ref = &mut *g.av.as_mut_ptr().add(g.processed);
+                if !f(item_mut_ref) {
+                    // update counters before drop_in_place, as it may panic
+                    g.processed += 1;
+                    g.deleted += 1;
+                    ptr::drop_in_place(item_mut_ref);
+                    continue;
+                } else {
+                    ptr::copy_nonoverlapping(
+                        item_mut_ref as *const _,
+                        g.av.as_mut_ptr().add(g.processed - g.deleted),
+                        1,
+                    );
+                }
+                g.processed += 1;
+            }
+        }
+    }
 }
 
 impl<T, L, SM, const C: usize> ArrayVec<T, L, SM, C>
